"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const config_1 = require("./../../modules/config");
const RELOAD_DELAY = 2000;
const watchedFiles = {};
// reloading changed files in development mode
function watchFileAndAct(file, handler) {
    file = path.normalize(file);
    if (!fs.existsSync(file)) {
        throw new Error(file + " does not exist, and cannot be watched");
    }
    if (config_1.config.general.developmentMode === true && !watchedFiles[file]) {
        const watcher = fs.watch(file, { persistent: false }, eventType => {
            const stats = fs.statSync(file);
            const delta = stats.mtimeMs - watchedFiles[file].stats.mtimeMs;
            if (config_1.config.general.developmentMode === true &&
                eventType === "change" &&
                delta >= RELOAD_DELAY) {
                watchedFiles[file].stats = stats;
                let cleanPath = file;
                if (process.platform === "win32") {
                    cleanPath = file.replace(/\//g, "\\");
                }
                delete require.cache[require.resolve(cleanPath)];
                handler(file);
            }
        });
        watchedFiles[file] = { watcher, stats: fs.statSync(file) };
    }
}
exports.watchFileAndAct = watchFileAndAct;
function unWatchAllFiles() {
    for (const file in watchedFiles) {
        watchedFiles[file].watcher.close();
        delete watchedFiles[file];
    }
}
exports.unWatchAllFiles = unWatchAllFiles;
